// 005 u_c_visitor_23
// 11 dec 2004

// -- Delphi implementation from the book:
// --   Design Patterns Elements of Reusable Object-Oriented Software
// --   Addison Wesley - 1994 - ISBN 0-201-63361-2
// -- (C) Erich GAMMA, Richard HELM, Ralph JOHNSON, John VLISSIDES 1994

// -- translation from C to Delphi performed by
// --   Felix John COLIBRI  http://www.felix-colibri.com

(*$r+*)

unit u_c_visitor_23;
  interface
    uses Classes;

    type t_watt= Double;
         t_currency= Double;

         c_equipment_visitor= Class; // forward

         // -- abstract
         c_equipment= class
                        _m_equipment_name: String;

                        Constructor create_equipment(p_equipment_name: String);
                        function f_equipment_name: String;
                        function f_power: t_watt; Virtual;
                        function f_net_price: t_currency; Virtual;
                        function f_discount_price: t_currency; Virtual;
                        procedure accept(p_c_equipment_visitor: c_equipment_visitor); Virtual;
                        Destructor Destroy; Override;
                      end; // c_equipment

         c_floppy= Class(c_equipment)
                     constructor create_floppy(p_floppy_name: String);
                     procedure accept(p_c_equipment_visitor: c_equipment_visitor); Override;
                   end; // c_floppy
         c_card= Class(c_equipment)
                end; // c_card
         c_chassis= Class(c_equipment)
                      _m_c_equipment_list: tList;
                      constructor create_chassis(p_chassis_name: String);
                      procedure accept(p_c_equipment_visitor: c_equipment_visitor); Override;
                    end; // c_chassis
         c_bus= Class(c_equipment)
                end; // c_bus

         c_equipment_visitor= Class
                                constructor create_equipment_visitor;
                                procedure visit_floppy(p_c_floppy: c_floppy); Virtual;
                                procedure visit_card(p_c_card: c_card); Virtual;
                                procedure visit_chassis(p_c_chassis: c_chassis); Virtual;
                                procedure visit_bus(p_c_bus: c_bus); Virtual;
                                Destructor Destroy; Override;
                              end; // c_equipment_visitor

         c_pricing_visitor= Class(c_equipment_visitor)
                              _total_price: t_currency;

                              constructor create_pricing_visitor;
                              procedure visit_floppy(p_c_floppy: c_floppy); Override;
                              procedure visit_card(p_c_card: c_card); Override;
                              procedure visit_chassis(p_c_chassis: c_chassis); Override;
                              procedure visit_bus(p_c_bus: c_bus); Override;

                              function f_net_price: t_currency;
                              Destructor Destroy; Override;
                            end; // c_pricing_visitor

         // -- a second visitor
         c_inventory= Class
                        procedure accumulate(p_equimpent: c_equipment);
                        function f_display_inventory: String;
                      end; // c_inventory
         c_inventory_visitor= Class(c_equipment_visitor)
                                _m_c_inventory: c_inventory;

                                constructor create_inventory_visitor;
                                procedure visit_floppy(p_c_floppy: c_floppy); Override;
                                procedure visit_card(p_c_card: c_card); Override;
                                procedure visit_chassis(p_c_chassis: c_chassis); Override;
                                procedure visit_bus(p_c_bus: c_bus); Override;

                                function f_c_inventory: c_inventory;
                                Destructor Destroy; Override;
                              end; // c_inventory_visitor

  implementation

    // -- c_equipment

    Constructor c_equipment.create_equipment(p_equipment_name: string);
      begin
      end; // create_equipment

    function c_equipment.f_equipment_name: string;
      begin
      end; // f_equipment_name

    function c_equipment.f_power: t_watt;
      begin
      end; // f_power

    function c_equipment.f_net_price: t_currency;
      begin
      end; // f_net_price

    function c_equipment.f_discount_price: t_currency;
      begin
      end; // f_discount_price

    procedure c_equipment.accept(p_c_equipment_visitor: c_equipment_visitor);
      begin
      end; // accept

    Destructor c_equipment.Destroy;
      begin
        Inherited;
      end; // Destroy

    // -- c_floppy

    constructor c_floppy.create_floppy(p_floppy_name: string);
      begin
      end; // create_floppy

    procedure c_floppy.accept(p_c_equipment_visitor: c_equipment_visitor);
      begin
        p_c_equipment_visitor.visit_floppy(Self);
      end; // accept

    // -- c_chassis

    constructor c_chassis.create_chassis(p_chassis_name: string);
      begin
      end; // create_chassis

    procedure c_chassis.accept(p_c_equipment_visitor: c_equipment_visitor);
      var l_equipment_index: Integer;
      begin
        for l_equipment_index:= 0 to _m_c_equipment_list.Count- 1 do
          c_equipment(_m_c_equipment_list[l_equipment_index]).accept(p_c_equipment_visitor);
        p_c_equipment_visitor.visit_chassis(Self);  
      end; // accept

    // -- c_equipment_visitor

    constructor c_equipment_visitor.create_equipment_visitor;
      begin
      end; // create_equipment_visitor

    procedure c_equipment_visitor.visit_floppy(p_c_floppy: c_floppy);
      begin
      end; // visit_floppy

    procedure c_equipment_visitor.visit_card(p_c_card: c_card);
      begin
      end; // visit_card

    procedure c_equipment_visitor.visit_chassis(p_c_chassis: c_chassis);
      begin
      end; // visit_chassis

    procedure c_equipment_visitor.visit_bus(p_c_bus: c_bus);
      begin
      end; // visit_bus

    Destructor c_equipment_visitor.Destroy;
      begin
      end; // Destroy

    // -- c_pricing_visitor

    constructor c_pricing_visitor.create_pricing_visitor;
      begin
      end; // create_pricing_visitor

    procedure c_pricing_visitor.visit_floppy(p_c_floppy: c_floppy);
      begin
        _total_price:= _total_price+ p_c_floppy.f_net_price;
      end; // visit_floppy

    procedure c_pricing_visitor.visit_card(p_c_card: c_card);
      begin
        // -- can change the pricing policy
        _total_price:= _total_price+ p_c_card.f_discount_price;
      end; // visit_card

    procedure c_pricing_visitor.visit_chassis(p_c_chassis: c_chassis);
      begin
      end; // visit_chassis

    procedure c_pricing_visitor.visit_bus(p_c_bus: c_bus);
      begin
      end; // visit_bus

    function c_pricing_visitor.f_net_price: t_currency;
      begin
      end; // f_net_price

    Destructor c_pricing_visitor.Destroy;
      begin
      end; // Destroy

    // -- c_inventory

    procedure c_inventory.accumulate(p_equimpent: c_equipment);
      begin
      end; // accumulate

    function c_inventory.f_display_inventory: String;
      begin
      end; // f_display_inventory

    // -- c_inventory_visitor

    constructor c_inventory_visitor.create_inventory_visitor;
      begin
      end; // create_inventory_visitor

    procedure c_inventory_visitor.visit_floppy(p_c_floppy: c_floppy);
      begin
        _m_c_inventory.accumulate(p_c_floppy);
      end; // visit_floppy

    procedure c_inventory_visitor.visit_card(p_c_card: c_card);
      begin
        _m_c_inventory.accumulate(p_c_card);
      end; // visit_card

    procedure c_inventory_visitor.visit_chassis(p_c_chassis: c_chassis);
      begin
      end; // visit_chassis

    procedure c_inventory_visitor.visit_bus(p_c_bus: c_bus);
      begin
      end; // visit_bus

    function c_inventory_visitor.f_c_inventory: c_inventory;
      begin
      end; // f_c_inventory

    Destructor c_inventory_visitor.Destroy;
      begin
      end; // Destroy

    begin // u_c_equipment
    end. // u_c_equipment
